"""An extension for discord.py that allows you to subclass `discord.Interaction` to add custom functionality."""  # noqa: I002

from .core import *
from .interactions import *
from .tree import *

__author__ = "Soheab"
__version__ = "0.0.1"
__license__ = "MPL-2.0"
